const Lang = imports.lang;
const St = imports.gi.St;
const Gio = imports.gi.Gio;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;
const Main = imports.ui.main;
const Params = imports.misc.params;
const Gettext = imports.gettext;
const Soup = imports.gi.Soup;
const Config = imports.misc.config;
const MessageTray = imports.ui.messageTray;
const ExtensionUtils = imports.misc.extensionUtils;
const Local = ExtensionUtils.getCurrentExtension();
const utils = Local.imports.utils



// Settings
const SETTINGS_SCHEMA = '/org/gnome/shell/extensions/hastebin-from-clipboard';
const POST_SHORTCUT_KEY = 'post-shortcut';


const clipboard = St.Clipboard.get_default();


function hastebinFromClipboard(clipboard_type) {
    clipboard.get_text(clipboard_type,
        Lang.bind(this, function(clipboard, text) {
            if(!text || /^\s*$/.test(text)) {
                Main.notify('Clipboard is empty.');
            }
            else {

                let _httpSession = new Soup.Session();
                let message = Soup.Message.new ("POST", "https://hastebin.com/documents");
                message.set_request ('text', Soup.MemoryUse.COPY, text);

                _httpSession.queue_message(message, Lang.bind(this, function(_httpSession, message) {
                    try {
                        if (!message.response_body.data) {
                            Main.notify("Unexpected error.");
                        }
                        let json_data = JSON.parse(message.response_body.data);
                        let key = json_data.key;
                        
                        let url = "http://hastebin.com/" + key;

                        let url_label = new St.Label({ text: url, style_class : "url"});

                        let source = new MessageTray.Source("Hastebin", 'edit-paste');
                        let notification = new MessageTray.Notification(source,
                                                'Hastebin uploader',
                                                "Sucess: " + url);

                        notification.addAction(_("Copy Link"), function () {
                            St.Clipboard.get_default().set_text(St.ClipboardType.PRIMARY, url);
                            St.Clipboard.get_default().set_text(St.ClipboardType.CLIPBOARD, url);
                        });
                        Main.messageTray.add(source);
                        source.notify(notification);

                    } catch (e) {
                        global.log("Error: " + response);
                        Main.notify("Error: " + response);
                    }
                }));
            }
        })
    );
}


function init() {

}

function enable() {
    let settings
    settings = utils.getSettings();

    Main.wm.addKeybinding(
            POST_SHORTCUT_KEY,
            settings,
            Meta.KeyBindingFlags.NONE,
            Shell.ActionMode.NORMAL,
            Lang.bind(this, function() {
                hastebinFromClipboard(St.ClipboardType.PRIMARY);
            })
        );
}

function disable() {
    Main.wm.removeKeybinding(POST_SHORTCUT_KEY)
}
