const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;
const ExtensionUtils = imports.misc.extensionUtils;
const Local = ExtensionUtils.getCurrentExtension();
const utils = Local.imports.utils;

const POST_SHORTCUT_KEY = 'post-shortcut';


function init() {
  _settings = utils.getSettings();
};


const HastebinSettingsWidget = new GObject.Class({
    Name: "HastebinSettingsWidget",
    GTypeName: "HastebinSettingsWidget",
    Extends: Gtk.Box,

    _init: function (params) {
        this.parent(params);

        let hbox;
        hbox = new Gtk.Box({
            orientation: Gtk.Orientation.HORIZONTAL,
            expand: false
        });

        const label = new Gtk.Label({
            label: 'Hotkey',
            xalign: 0,
            expand: true
        });

        hbox.add(label, {fil: false});

        let model = new Gtk.ListStore();

        model.set_column_types([
            GObject.TYPE_STRING,
            GObject.TYPE_STRING,
            GObject.TYPE_INT,
            GObject.TYPE_INT
        ]);

        let binding = _settings.get_strv(POST_SHORTCUT_KEY)[0]
        let key, mods
        [key, mods] = Gtk.accelerator_parse(binding);

        let row = model.append();
        model.set(row, [0, 1, 2, 3], ["post-shortcut", "Hotkey", mods, key]);

        let treeview = new Gtk.TreeView({
            'expand': true,
            'model': model
        });

        let cellrend = new Gtk.CellRendererText();
        let col = new Gtk.TreeViewColumn({'title': 'Keyboard Shortcut'});
        col.pack_start(cellrend, true);
        col.add_attribute(cellrend, 'text', 1);
        treeview.append_column(col);

        hotkey = new Gtk.CellRendererAccel({
            'editable': true,
            'accel-mode': Gtk.CellRendererAccelMode.GTK
        });

        hotkey.connect('accel-edited', function(rend, iter, key, mods) {
            let value = Gtk.accelerator_name(key, mods);
            let [succ, iterator] = model.get_iter_from_string(iter);

            if (!succ) {
                throw new Error("Error updating keybinding");
            }

            let name = model.get_value(iterator, 0);

            model.set(iterator, [2, 3], [mods, key]);
            _settings.set_strv(name, [value]);
        });

        hotkey.connect('accel-cleared', function(rend, iter, key, mods) {
            let [succ, iterator] = model.get_iter_from_string(iter);

            if (!succ) {
                throw new Error("Error clearing keybinding");
            }

            let name = model.get_value(iterator, 0);

            model.set(iterator, [2, 3], [0, 0]);
            _settings.set_strv(name, []);
        });

        col = new Gtk.TreeViewColumn({'title': 'Modify', min_width: 200});
        col.pack_end(hotkey, false);
        col.add_attribute(hotkey, 'accel-mods', 2);
        col.add_attribute(hotkey, 'accel-key', 3);
        treeview.append_column(col);

        this.add(treeview);
    },

});

function buildPrefsWidget() {
    let widget = new HastebinSettingsWidget();
    widget.show_all();
    return widget;
};